from airunner.gui.windows.setup_wizard.base_wizard import BaseWizard
from airunner.gui.windows.setup_wizard.installation_settings.templates.install_failed_page_ui import Ui_install_failed_page


class InstallFailedPage(BaseWizard):
    class_name_ = Ui_install_failed_page

