
from airunner.gui.widgets.base_widget import BaseWidget
from airunner.gui.widgets.batch.templates.batch_widget_ui import Ui_batch_widget


class BatchWidget(BaseWidget):
    widget_class_ = Ui_batch_widget
