from airunner.settings import AIRUNNER_ART_ENABLED


controlnet_bootstrap_data = [
    {
        "display_name": "Canny",
        "name": "canny",
        "path": "lllyasviel/control_v11p_sd15_canny",
        "version": "SD 1.5",
        "pipeline_action": "controlnet",
        "size": "722600",
    },
    {
        "display_name": "Depth",
        "name": "depth_leres",
        "path": "lllyasviel/control_v11f1p_sd15_depth",
        "version": "SD 1.5",
        "pipeline_action": "controlnet",
        "size": "722600",
    },
    {
        "display_name": "MLSD",
        "name": "mlsd",
        "path": "lllyasviel/control_v11p_sd15_mlsd",
        "version": "SD 1.5",
        "pipeline_action": "controlnet",
        "size": "722600",
    },
    {
        "display_name": "Normal",
        "name": "normal_bae",
        "path": "lllyasviel/control_v11p_sd15_normalbae",
        "version": "SD 1.5",
        "pipeline_action": "controlnet",
        "size": "722600",
    },
    {
        "display_name": "Scribble",
        "name": "scribble_hed",
        "path": "lllyasviel/control_v11p_sd15_scribble",
        "version": "SD 1.5",
        "pipeline_action": "controlnet",
        "size": "722600",
    },
    {
        "display_name": "Segmentation",
        "name": "segmentation",
        "path": "lllyasviel/control_v11p_sd15_seg",
        "version": "SD 1.5",
        "pipeline_action": "controlnet",
        "size": "722600",
    },
    {
        "display_name": "Lineart",
        "name": "lineart_coarse",
        "path": "lllyasviel/control_v11p_sd15_lineart",
        "version": "SD 1.5",
        "pipeline_action": "controlnet",
        "size": "722600",
    },
    {
        "display_name": "Lineart Anime",
        "name": "lineart_anime",
        "path": "lllyasviel/control_v11p_sd15s2_lineart_anime",
        "version": "SD 1.5",
        "pipeline_action": "controlnet",
        "size": "722600",
    },
    {
        "display_name": "Openpose",
        "name": "openpose",
        "path": "lllyasviel/control_v11p_sd15_openpose",
        "version": "SD 1.5",
        "pipeline_action": "controlnet",
        "size": "722600",
    },
    {
        "display_name": "Softedge",
        "name": "softedge_hed",
        "path": "lllyasviel/control_v11p_sd15_softedge",
        "version": "SD 1.5",
        "pipeline_action": "controlnet",
        "size": "722600",
    },
    {
        "display_name": "Pixel2Pixel",
        "name": "pixel2pixel",
        "path": "lllyasviel/control_v11e_sd15_ip2p",
        "version": "SD 1.5",
        "pipeline_action": "controlnet",
        "size": "722600",
    },
    {
        "display_name": "Inpaint",
        "name": "inpaint",
        "path": "lllyasviel/control_v11p_sd15_inpaint",
        "version": "SD 1.5",
        "pipeline_action": "controlnet",
        "size": "722600",
    },
    {
        "display_name": "Shuffle",
        "name": "shuffle",
        "path": "lllyasviel/control_v11e_sd15_shuffle",
        "version": "SD 1.5",
        "pipeline_action": "controlnet",
        "size": "722600",
    },
    {
        "display_name": "Canny SDXL",
        "name": "canny",
        "path": "diffusers/controlnet-canny-sdxl-1.0-small",
        "version": "SDXL 1.0",
        "pipeline_action": "controlnet",
        "size": "320200",
    },
]


if not AIRUNNER_ART_ENABLED:
    controlnet_bootstrap_data = []

